( cd usr/lib ; rm -rf libsox.so )
( cd usr/lib ; ln -sf libsox.so.1.0.0 libsox.so )
( cd usr/lib ; rm -rf libsox.so.1 )
( cd usr/lib ; ln -sf libsox.so.1.0.0 libsox.so.1 )
( cd usr/man/man1 ; rm -rf play.1.gz )
( cd usr/man/man1 ; ln -sf sox.1.gz play.1.gz )
( cd usr/man/man1 ; rm -rf rec.1.gz )
( cd usr/man/man1 ; ln -sf sox.1.gz rec.1.gz )
( cd usr/man/man7 ; rm -rf soxeffect.7.gz )
( cd usr/man/man7 ; ln -sf ../man1/sox.1.gz soxeffect.7.gz )
( cd usr/bin ; rm -rf rec )
( cd usr/bin ; ln -sf sox rec )
( cd usr/bin ; rm -rf play )
( cd usr/bin ; ln -sf sox play )
( cd usr/bin ; rm -rf soxi )
( cd usr/bin ; ln -sf sox soxi )
